#include "headtrackingcamera.h"

namespace Msp {
namespace VR {

HeadTrackingCamera::HeadTrackingCamera(const GL::Camera &c):
	base_camera(c),
	elevation_offset(0)
{ }

void HeadTrackingCamera::set_elevation_offset(float o)
{
	elevation_offset = o;
}

void HeadTrackingCamera::update_from_matrix(const GL::Matrix &hmd_matrix)
{
	const GL::Vector3 &up = base_camera.get_up_direction();
	GL::Vector3 right = normalize(cross(base_camera.get_look_direction(), up));
	GL::Vector4 columns[4];
	columns[0] = compose(right, 0.0f);
	columns[1] = compose(up, 0.0f);
	columns[2] = compose(cross(right, up), 0.0f);
	columns[3] = compose(base_camera.get_position(), 1.0f);
	GL::Matrix matrix = GL::Matrix::from_columns(columns)*hmd_matrix;

	set_position(matrix*GL::Vector3()+base_camera.get_up_direction()*elevation_offset);
	set_look_direction((matrix*GL::Vector4(0, 0, -1, 0)).slice<3>(0));
	set_up_direction((matrix*GL::Vector4(0, 1, 0, 0)).slice<3>(0));

	set_depth_clip(base_camera.get_near_clip(), base_camera.get_far_clip());
	set_field_of_view(base_camera.get_field_of_view());
	set_aspect(base_camera.get_aspect());
}

} // namespace VR
} // namespace Msp
